unit mainformu;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Data.FireDACJSONReflect,
  FireDAC.Stan.Intf, FireDAC.Stan.Option, FireDAC.Stan.Param,
  FireDAC.Stan.Error, FireDAC.DatS, FireDAC.Phys.Intf, FireDAC.DApt.Intf,
  Data.DBXDataSnap, IPPeerClient, Data.DBXCommon, Data.DB, Data.SqlExpr,
  FireDAC.Comp.DataSet, FireDAC.Comp.Client, Vcl.StdCtrls, Vcl.Grids,
  Vcl.DBGrids, ServerMethodsClientu,
  FireDAC.Stan.StorageBin, FireDAC.UI.Intf, FireDAC.VCLUI.Wait, FireDAC.Comp.UI;

type
  TForm2 = class(TForm)
    FDMemTableCustomers: TFDMemTable;
    DBGrid1: TDBGrid;
    DataSource1: TDataSource;
    ReadFromServerBtn: TButton;
    SQLConnection1: TSQLConnection;
    FDStanStorageBinLink1: TFDStanStorageBinLink;
    FDGUIxWaitCursor1: TFDGUIxWaitCursor;
    ApplyUpdatesBtn: TButton;
    ConnectBtn: TButton;
    procedure ReadFromServerBtnClick(Sender: TObject);
    procedure DataSource1DataChange(Sender: TObject; Field: TField);
    procedure ApplyUpdatesBtnClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure ConnectBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    function Connect: TServerMethods1Client;
    procedure GetCustomerQuery;
    procedure GetCustomerQueryJSON;
    procedure ApplyCustomerUpdates;
    procedure ApplyCustomerUpdatesJSON;
  end;

var
  Form2: TForm2;

implementation

{$R *.dfm}

uses System.JSON;

var
  ServerMethods1Client: TServerMethods1Client;



{ TForm2 }

procedure TForm2.ReadFromServerBtnClick(Sender: TObject);
begin
  GetCustomerQueryJSON;
  ReadFromServerBtn.Enabled := not FDMemTableCustomers.Active;
end;

procedure TForm2.ApplyCustomerUpdates;
var
  Deltas: TFDJSONDeltas;
begin

  if FDMemTableCustomers.State in dsEditModes then
  begin
    FDMemTableCustomers.Post;
  end;
  // Create a delta list
  Deltas := TFDJSONDeltas.Create;
  // Add deltas
  TFDJSONDeltasWriter.ListAdd(Deltas, 'CustomerQuery', FDMemTableCustomers);
    // Call server method.  Pass the delta list.
  ServerMethods1Client.ApplyCustomerUpdates(Deltas);
end;

procedure TForm2.ApplyCustomerUpdatesJSON;
var
  Deltas: TFDJSONDeltas;
  JSONObject: TJSONObject;
begin
  // Create a delta list
  Deltas := TFDJSONDeltas.Create;
  // Create the JSONObject
  JSONObject := TJSONObject.Create;
  // Add deltas
  TFDJSONDeltasWriter.ListAdd(Deltas, 'CustomerQuery', FDMemTableCustomers);
  TFDJSONInterceptor.DataSetsToJSONObject(Deltas, JSONObject);
    // Call server method.  Pass the JSONObject list.
  ServerMethods1Client.ApplyCustomerUpdatesJSON(JSONObject);
end;

procedure TForm2.ApplyUpdatesBtnClick(Sender: TObject);
begin
  ApplyCustomerUpdates;
end;

procedure TForm2.Button1Click(Sender: TObject);
begin
  ShowMessage(BoolToStr( FDMemTableCustomers.CachedUpdates, True));
  ShowMessage(BoolToStr( FDMemTableCustomers.UpdatesPending, True));
  ShowMessage(IntToStr( FDMemTableCustomers.ChangeCount));
end;

procedure TForm2.ConnectBtnClick(Sender: TObject);
begin
  if ConnectBtn.Caption = 'Connect' then
  begin
    try
      SQLConnection1.Open;
    except
      on e: Exception do
      begin
        ShowMessage(e.Message);
        exit;
      end;
    end;
    Connect;
    ConnectBtn.Caption := 'Disconnect';
    ReadFromServerBtn.Enabled := True;

  end
  else
  begin
    SQLConnection1.Close;
    FDMemTableCustomers.Close;
    ReadFromServerBtn.Enabled := False;
    ApplyUpdatesBtn.Enabled := False;
    ServerMethods1Client := nil;
    ConnectBtn.Caption := 'Connect';
  end;
end;

function TForm2.Connect: TServerMethods1Client;
begin
  if ServerMethods1Client = nil then
    ServerMethods1Client := TServerMethods1Client.Create(SQLConnection1.DBXConnection);
  Result := ServerMethods1Client;
end;

procedure TForm2.DataSource1DataChange(Sender: TObject; Field: TField);
begin
  ApplyUpdatesBtn.Enabled := FDMemTableCustomers.UpdatesPending;
end;

procedure TForm2.GetCustomerQuery;
var
  FDJSONDataSets: TFDJSONDataSets;
begin
  FDJSONDataSets := ServerMethods1Client.GetCustomerData;
  FDMemTableCustomers.Active  := False;
  FDMemTableCustomers.CachedUpdates := False;
  FDMemTableCustomers.Data := TFDJSONDataSetsReader.GetListValue(FDJSONDataSets, 0).Data;
  FDMemTableCustomers.CommitUpdates;
  FDMemTableCustomers.CachedUpdates := True;
end;

procedure TForm2.GetCustomerQueryJSON;
var
  JSONObject: TJSONObject;
  FDJSONDataSets: TFDJSONDataSets;
  ItemsList: TFDJSONInterceptor.TItemList;
begin
  JSONObject := ServerMethods1Client.GetCustomerDataJSON;
  FDJSONDataSets := TFDJSONDataSets.Create;
  if TFDJSONInterceptor.JSONObjectToDataSets(JSONObject, FDJSONDataSets) then
    ShowMessage('True')
    else
    ShowMessage('False');

  FDMemTableCustomers.Active  := False;
  FDMemTableCustomers.CachedUpdates := False;
  FDMemTableCustomers.Data := TFDJSONDataSetsReader.GetListValue(FDJSONDataSets, 0).Data;
  FDMemTableCustomers.CommitUpdates;
  FDMemTableCustomers.CachedUpdates := True;
end;

end.
