unit ServerMethodsUnit1;

interface

uses System.SysUtils, System.Classes, System.Json,
    Datasnap.DSServer, Datasnap.DSAuth, DataSnap.DSProviderDataModuleAdapter,
  FireDAC.Stan.Intf, FireDAC.Stan.Option, FireDAC.Stan.Error, FireDAC.UI.Intf,
  FireDAC.Phys.Intf, FireDAC.Stan.Def, FireDAC.Stan.Pool, FireDAC.Stan.Async,
  FireDAC.Phys, FireDAC.Stan.Param, FireDAC.DatS, FireDAC.DApt.Intf,
  FireDAC.DApt, Data.DB, FireDAC.Comp.DataSet, FireDAC.Comp.Client,
  Data.FireDACJSONReflect, FireDAC.Phys.IB, FireDAC.Phys.IBDef,
  FireDAC.Stan.StorageJSON;

type
{$METHODINFO ON}
  TServerMethods1 = class(TDataModule)
    FDConnection1: TFDConnection;
    CustomerQuery: TFDQuery;
    procedure DataModuleCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    function GetCustomerData: TFDJSONDataSets;
    function GetCustomerDataJSON: TJSONObject;
    procedure ApplyCustomerUpdates(const ADeltaList: TFDJSONDeltas);
    procedure ApplyCustomerUpdatesJSON(const JSONObject: TJSONObject);
  end;
{$METHODINFO OFF}

implementation


{$R *.dfm}


uses System.StrUtils, dialogs;

{ TServerMethods1 }

procedure TServerMethods1.ApplyCustomerUpdatesJSON(const JSONObject: TJSONObject);
var
  Deltas: TFDJSONDeltas;
  DeltasApplyUpdates: IFDJSONDeltasApplyUpdates;
begin
  Deltas := TFDJSONDeltas.Create;
  TFDJSONInterceptor.JSONObjectToDataSets(JSONObject, Deltas);
  DeltasApplyUpdates := TFDJSONDeltasApplyUpdates.Create(Deltas);
  // Apply the department delta
  DeltasApplyUpdates.ApplyUpdates(0, CustomerQuery.Command);
  if DeltasApplyUpdates.Errors.Count > 0 then
    // Raise an exception if any errors.
    raise Exception.Create(DeltasApplyUpdates.Errors.Strings.Text);
end;

procedure TServerMethods1.ApplyCustomerUpdates(const ADeltaList: TFDJSONDeltas);
var
  DeltasApplyUpdates: IFDJSONDeltasApplyUpdates;
begin
  DeltasApplyUpdates := TFDJSONDeltasApplyUpdates.Create(ADeltaList);
  DeltasApplyUpdates.Count;
  DeltasApplyUpdates.ApplyUpdates(0, CustomerQuery.Command);
  if DeltasApplyUpdates.Errors.Count > 0 then
    raise Exception.Create(DeltasApplyUpdates.Errors.Strings.Text);
end;

procedure TServerMethods1.DataModuleCreate(Sender: TObject);
var
  DataDir: string;
begin
  DataDir := GetEnvironmentVariable('DEMOSDIR');
  if DataDir.IsEmpty then
  begin
    ShowMessage('DEMOSDIR environment variable is not set. ' +
                'Cannot continue');
    exit;
  end;

  FDConnection1.Params.Clear;
  FDConnection1.Params.Add('DriverID=IB');
  FDConnection1.Params.Add('Database=' +
    GetEnvironmentVariable('DEMOSDIR') + 'Data\dbdemos.gdb');
  FDConnection1.Params.Add('User_Name=sysdba');
  FDConnection1.Params.Add('password=masterkey');
  FDConnection1.Params.Add('Server=localhost');
  FDCOnnection1.LoginPrompt := False;
  FDConnection1.Open();
end;

function TServerMethods1.GetCustomerDataJSON: TJSONObject;
var
  DataSets: TFDJSONDataSets;
begin
  CustomerQuery.Active := False;
  DataSets := TFDJSONDataSets.Create;
  CustomerQuery.Open;
  CustomerQuery.FetchAll;
  try
    TFDJSONDataSetsWriter.ListAdd(DataSets, CustomerQuery);
    Result := TJSONObject.Create;
    TFDJSONInterceptor.DataSetsToJSONObject(DataSets, Result);
    CustomerQuery.Close;
  finally
    DataSets.Free;
  end;
end;

function TServerMethods1.GetCustomerData: TFDJSONDataSets;
begin
  CustomerQuery.Active := False;
  CustomerQuery.Open();
  CustomerQuery.FetchAll;
  Result := TFDJSONDataSets.Create;
  TFDJSONDataSetsWriter.ListAdd(Result, 'CustomerQuery', CustomerQuery);
  CustomerQuery.Close;
end;

end.

